/* arch/arm/mach-msm/kcj_reset_info.c
 *
 * Kyocera Reset Information
 *
 * Copyright (C) 2011 KYOCERA Corporation
 *
 * This software is licensed under the terms of the GNU General Public
 * License version 2, as published by the Free Software Foundation, and
 * may be copied, distributed, and modified under those terms.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 */
#include <linux/version.h>
#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/types.h>
#include <linux/time.h>
#include <linux/ctype.h>
#include <linux/console.h>
#include <linux/miscdevice.h>
#include <linux/fs.h>
#include <linux/ptrace.h>
#include <linux/spinlock.h>
#include <linux/init.h>
#include <asm/uaccess.h>
#include <asm/ioctl.h>
#include <asm/cputime.h>
#include <mach/msm_iomap.h>
#include <kcj_reset_info.h>

MODULE_LICENSE("GPL v2");

/****************************************************************************
 * Local Variable/Structure Declaration                                     
 ****************************************************************************/
spinlock_t           g_rstinfo_lock;
static t_RESET_INFO *l_rstinfo = (void *)(RSTINFO_START_ADDR);
static t_RESET_INFO  g_rstinfo_tmp;

/****************************************************************************
 *  ֐   : unsigned long rstinfo_calc_checksum                          
 *  @\     : WX^񏉊RAMɕۑ                          
 *       : (I) int  update  true :checksuml̍XV                      
 *           :                  false:checksuml̏ƍ                      
 *  ߂l   : update = false̎                                           
 *           :   true :I                                             
 *           :   false:ُI                                             
 *           : update = true̎                                            
 *           :   checksuml                                                 
 *  l     : Ȃ                                                         
 ****************************************************************************/
static unsigned long rstinfo_calc_checksum
(
    int update
)
{
    unsigned long  sum;
    unsigned long  i;
    unsigned char *p;

    p = (unsigned char*)l_rstinfo;

    /* calc sum */
    for (i = 0, sum = 0; i<sizeof(t_RESET_INFO) - sizeof(unsigned long);i++, p++)
    {
        sum += *p;
    }

    if (update == false)
    {
        /* checksuml̏ƍ */
        if (l_rstinfo->sum != sum)
        {
            printk(KERN_INFO "[OTA-DIAG]CHECKSUM ERROR\n");
            return false;
        }
        else
        {
            return true;
        }
    }
    else
    {
        /* checksuml̍XV */
        l_rstinfo->sum = sum;
        return sum;
    }
}

/****************************************************************************
 *  ֐   : int rstinfo_save                                             
 *  @\     : WX^񏉊RAMɕۑ                          
 *       : (I) struct pt_regs  *regs  WX^                      
 *  ߂l   : Ȃ                                                         
 *  l     : Ȃ                                                         
 ****************************************************************************/
void rstinfo_save
(
    struct pt_regs *regs
)
{
    struct  timespec  now;
    struct  timespec  uptime;
    struct  timespec  idletime;
            cputime_t cpu_idletime;

    /* p[^`FbN */
    if (regs == NULL)
    {
        printk(KERN_ERR "[RSTINFO]regs == NUL\n");
        return;
    }

    /* Get Time */
    now          = current_kernel_time();
    cpu_idletime = cputime_add(init_task.utime, init_task.stime);
    do_posix_clock_monotonic_gettime(&uptime);
    monotonic_to_bootbased(&uptime);
    cputime_to_timespec(cpu_idletime, &idletime);

    /* XsbN */
    spin_lock(&g_rstinfo_lock);

    /* 񏉊RAM̏ */
    memset(l_rstinfo, 0, sizeof(t_RESET_INFO));

    /* Save Registers */
    memcpy((void *)&(l_rstinfo->regs), (void *)regs, sizeof(struct pt_regs));

    /* Magic */
    memcpy(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN);

    /* Time */
    l_rstinfo->crashtime.tv_sec  = now.tv_sec;
    l_rstinfo->crashtime.tv_nsec = now.tv_nsec;
    l_rstinfo->uptime.tv_sec     = uptime.tv_sec;
    l_rstinfo->uptime.tv_nsec    = uptime.tv_nsec;
    l_rstinfo->idletime.tv_sec   = idletime.tv_sec;
    l_rstinfo->idletime.tv_nsec  = idletime.tv_nsec;

    l_rstinfo->cause     = RSTINFO_UNEXPECTED_EXCEPTION;
    l_rstinfo->processor = RSTINFO_PROCESSOR_APPLICATION;

    l_rstinfo->remaining = 0;

    /* Save Stack */
    if (user_mode(regs))
    {
        /* UserMode */
        printk(KERN_DEBUG "[RSTINFO]MODE=USER\n");

        if (copy_from_user((void *)&(l_rstinfo->stacks), (void *)((regs->ARM_sp) &= ~0x03), sizeof(l_rstinfo->stacks)))
        {
            /* 񏉊RAM̏ */
            printk(KERN_ERR "[RSTINFO]Fail to SAVE RSTINFO[%d]", __LINE__);
            memset((void *)&(l_rstinfo->stacks), 0, sizeof(l_rstinfo->stacks));
        }
    }
    else
    {
        /* KernelMode*/
        struct thread_info *thread;
        struct task_struct *tsk;
        unsigned long       stack_limit;
        unsigned long       dumpsize;

        printk(KERN_DEBUG "[RSTINFO]MODE=KERNEL\n");

        /* StackDump Begin */
        l_rstinfo->dump_begin = ((regs->ARM_sp) &= ~0x03) - 64;

        /* Get StackLimit */
        thread = current_thread_info();
        tsk = thread->task;
        stack_limit = (unsigned long)(thread + 1);
        l_rstinfo->stack_limit = stack_limit;

        dumpsize = THREAD_SIZE + (unsigned long)task_stack_page(tsk) - l_rstinfo->dump_begin;

        if (dumpsize > sizeof(l_rstinfo->stacks))
        {
            dumpsize = sizeof(l_rstinfo->stacks);
        }

        memcpy((void *)&(l_rstinfo->stacks), (void *)l_rstinfo->dump_begin, dumpsize);
    }

    /* 񏉊RAMchecksumvZ */
    rstinfo_calc_checksum(true);

    /* XsbN */
    spin_unlock(&g_rstinfo_lock);

#if 0
printk(KERN_ERR "### ---------------------------------------- ###\n");
printk(KERN_ERR "### cause    =[%d] ###\n", l_rstinfo->cause);
printk(KERN_ERR "### processor=[%d] ###\n", l_rstinfo->processor);
printk(KERN_ERR "### remaining=[%d] ###\n", l_rstinfo->remaining);
printk(KERN_ERR "### uregs[ 0]=[%lx] ###\n", l_rstinfo->regs.uregs[0]);
printk(KERN_ERR "### uregs[ 1]=[%lx] ###\n", l_rstinfo->regs.uregs[1]);
printk(KERN_ERR "### uregs[ 2]=[%lx] ###\n", l_rstinfo->regs.uregs[2]);
printk(KERN_ERR "### uregs[ 3]=[%lx] ###\n", l_rstinfo->regs.uregs[3]);
printk(KERN_ERR "### uregs[ 4]=[%lx] ###\n", l_rstinfo->regs.uregs[4]);
printk(KERN_ERR "### uregs[ 5]=[%lx] ###\n", l_rstinfo->regs.uregs[5]);
printk(KERN_ERR "### uregs[ 6]=[%lx] ###\n", l_rstinfo->regs.uregs[6]);
printk(KERN_ERR "### uregs[ 7]=[%lx] ###\n", l_rstinfo->regs.uregs[7]);
printk(KERN_ERR "### uregs[ 8]=[%lx] ###\n", l_rstinfo->regs.uregs[8]);
printk(KERN_ERR "### uregs[ 9]=[%lx] ###\n", l_rstinfo->regs.uregs[9]);
printk(KERN_ERR "### uregs[10]=[%lx] ###\n", l_rstinfo->regs.uregs[10]);
printk(KERN_ERR "### uregs[11]=[%lx] ###\n", l_rstinfo->regs.uregs[11]);
printk(KERN_ERR "### uregs[12]=[%lx] ###\n", l_rstinfo->regs.uregs[12]);
printk(KERN_ERR "### uregs[13]=[%lx] ###\n", l_rstinfo->regs.uregs[13]);
printk(KERN_ERR "### uregs[14]=[%lx] ###\n", l_rstinfo->regs.uregs[14]);
printk(KERN_ERR "### uregs[15]=[%lx] ###\n", l_rstinfo->regs.uregs[15]);
printk(KERN_ERR "### uregs[16]=[%lx] ###\n", l_rstinfo->regs.uregs[16]);
printk(KERN_ERR "### uregs[17]=[%lx] ###\n", l_rstinfo->regs.uregs[17]);
printk(KERN_ERR "### ---------------------------------------- ###\n");
#endif

    return;
}
EXPORT_SYMBOL(rstinfo_save);

/****************************************************************************
 *  ֐   : int rstinfo_save_stack                                       
 *  @\     : X^bN񏉊RAMɕۑ                          
 *       : (I) struct pt_regs  *regs  WX^                      
 *  ߂l   : Ȃ                                                         
 *  l     : Ȃ                                                         
 ****************************************************************************/
static void rstinfo_save_stack
(
    t_RESET_INFO *data
)
{
    struct  timespec  now;
    struct  timespec  uptime;
    struct  timespec  idletime;
            cputime_t cpu_idletime;

    /* p[^`FbN */
    if (data == NULL)
    {
        printk(KERN_ERR "[RSTINFO]data == NUL\n");
        return;
    }

    /* Get Time */
    now          = current_kernel_time();
    cpu_idletime = cputime_add(init_task.utime, init_task.stime);
    do_posix_clock_monotonic_gettime(&uptime);
    monotonic_to_bootbased(&uptime);
    cputime_to_timespec(cpu_idletime, &idletime);

    /* XsbN */
    spin_lock(&g_rstinfo_lock);

    /* 񏉊RAM̏ */
    memset(l_rstinfo, 0, sizeof(t_RESET_INFO));

    /* Magic */
    memcpy(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN);

    /* Time */
    l_rstinfo->crashtime.tv_sec  = now.tv_sec;
    l_rstinfo->crashtime.tv_nsec = now.tv_nsec;
    l_rstinfo->uptime.tv_sec     = uptime.tv_sec;
    l_rstinfo->uptime.tv_nsec    = uptime.tv_nsec;
    l_rstinfo->idletime.tv_sec   = idletime.tv_sec;
    l_rstinfo->idletime.tv_nsec  = idletime.tv_nsec;

    l_rstinfo->processor = RSTINFO_PROCESSOR_APPLICATION;
    l_rstinfo->cause     = data->cause;

    l_rstinfo->remaining = 0;
    memcpy(l_rstinfo->stacks, data->stacks, RSTINFO_STACK_SIZE);

    /* 񏉊RAMchecksumvZ */
    rstinfo_calc_checksum(true);

    /* XsbN */
    spin_unlock(&g_rstinfo_lock);

#if 0
printk(KERN_ERR "### ---------------------------------------- ###\n");
printk(KERN_ERR "### cause    =[%d] ###\n", l_rstinfo->cause);
printk(KERN_ERR "### processor=[%d] ###\n", l_rstinfo->processor);
printk(KERN_ERR "### remaining=[%d] ###\n", l_rstinfo->remaining);
printk(KERN_ERR "### stacks[0]=[%c] ###\n", l_rstinfo->stacks[0]);
printk(KERN_ERR "### stacks[1]=[%c] ###\n", l_rstinfo->stacks[1]);
printk(KERN_ERR "### stacks[2]=[%c] ###\n", l_rstinfo->stacks[2]);
printk(KERN_ERR "### stacks[3]=[%c] ###\n", l_rstinfo->stacks[3]);
printk(KERN_ERR "### stacks[4]=[%c] ###\n", l_rstinfo->stacks[4]);
printk(KERN_ERR "### ---------------------------------------- ###\n");
#endif

    return;
}

/****************************************************************************
 *  ֐   : int rstinfo_ioctl                                            
 *  @\     : 񏉊RAM胊Zbgǂݍ                        
 *       : (I) struct inode  *inode  gp                             
 *           : (I) struct file   *filp   gp                             
 *           : (I) unsigned int   cmd    REQUEST_RESET_INFO:ResetǍ 
 *           :                           REQUEST_SAVE_STACK:Stack̕ۑ 
 *           :                           REQUEST_READ_MAGIC:MagicNo̓Ǎ   
 *           : (O) unsigned long  arg    Reseti[̈̃AhX  
 *  ߂l   : 0        I                                            
 *           : -EINVAL  ُ                                            
 *           : -ENOENT  f[^Ȃ                                          
 *           : -EFAULT  ُI                                            
 *  l     : Ȃ                                                         
 ****************************************************************************/
int rstinfo_ioctl
(
    struct inode  *inode,
    struct file   *filp,
    unsigned int   cmd,
    unsigned long  arg
)
{
    switch (cmd)
    {
        case REQUEST_RESET_INFO:

            printk(KERN_ERR "[RSTINFO]REQUEST_RESET_INFO\n");

            /* ǂݍݑOɃf[^̗LmF */
            if (strncmp(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN))
            {
                printk(KERN_ERR "[RSTINFO]No Data[%d]", __LINE__);
                return -ENOENT;
            }

            /* 񏉊RAM̓e[ÜɃRs[ */
            if (copy_to_user((void *)arg, (void *)l_rstinfo , sizeof(t_RESET_INFO)))
            {
                printk(KERN_ERR "[RSTINFO]Fail to SAVE RSTINFO[%d]", __LINE__);
                return -EFAULT;
            }

#if 0
printk(KERN_ERR "### ---------------------------------------- ###\n");
printk(KERN_ERR "### cause    =[%d] ###\n", l_rstinfo->cause);
printk(KERN_ERR "### processor=[%d] ###\n", l_rstinfo->processor);
printk(KERN_ERR "### remaining=[%d] ###\n", l_rstinfo->remaining);
printk(KERN_ERR "### stacks[0]=[%c] ###\n", l_rstinfo->stacks[0]);
printk(KERN_ERR "### stacks[1]=[%c] ###\n", l_rstinfo->stacks[1]);
printk(KERN_ERR "### stacks[2]=[%c] ###\n", l_rstinfo->stacks[2]);
printk(KERN_ERR "### stacks[3]=[%c] ###\n", l_rstinfo->stacks[3]);
printk(KERN_ERR "### stacks[4]=[%c] ###\n", l_rstinfo->stacks[4]);
printk(KERN_ERR "### ---------------------------------------- ###\n");
#endif

            /* 񏉊RAM */
            spin_lock(&g_rstinfo_lock);
            memset(l_rstinfo, 0, sizeof(t_RESET_INFO));
            spin_unlock(&g_rstinfo_lock);
            break;

        case REQUEST_SAVE_STACK:

            printk(KERN_ERR "[RSTINFO]REQUEST_SAVE_STACK\n");

            /* [Ü̓e񏉊RAMɃRs[ */
            if (copy_from_user((void *)&g_rstinfo_tmp, (void *)arg, sizeof(t_RESET_INFO)))
            {
                printk(KERN_ERR "[RSTINFO]Fail to SAVE RSTINFO[%d]", __LINE__);
                return -EFAULT;
            }

            /* X^bN̕ۑ */
            rstinfo_save_stack(&g_rstinfo_tmp);
            break;

        case REQUEST_READ_MAGIC:

            printk(KERN_ERR "[RSTINFO]REQUEST_READ_MAGIC\n");

            /* f[^̗LmF */
            if (strncmp(l_rstinfo->magic, "RSTINFO", RSTINFO_MAGIC_LEN))
            {
                printk(KERN_ERR "[RSTINFO]No Data[%d]", __LINE__);
                return -ENOENT;
            }
            break;

        default:

            printk(KERN_ERR "[RSTINFO]Cmd Error\n");

            return -EINVAL;
    }

    /* I */
    return 0;
}

static const struct file_operations t_reset_info = {
    .owner = THIS_MODULE,
    .ioctl = rstinfo_ioctl,
};

static struct miscdevice t_rstinfo_miscdev = {
    .minor = RSTINFO_MINOR,
    .name  = "rstinfo",
    .fops  = &t_reset_info,
};

/****************************************************************************
 *  ֐   : int rstinfo_init                                             
 *  @\     : foCXo^                                           
 *       : Ȃ                                                         
 *  ߂l   : 0      I                                              
 *           : 0ȊO  ُI(Negative Errno Code)                         
 *  l     : Ȃ                                                         
 ****************************************************************************/
static int __init rstinfo_init(void)
{
    int retval = 0;

    /* foCX̓o^ */
    retval = misc_register(&t_rstinfo_miscdev);

    if (retval) {
        /* foCX̍폜 */
        misc_deregister(&t_rstinfo_miscdev);
        return retval;
    }

    /* XsbN */
    spin_lock_init(&g_rstinfo_lock);

    return retval;
}

/****************************************************************************
 *  ֐   : void rstinfo_exit                                             
 *  @\     : foCX폜                                           
 *       : Ȃ                                                         
 *  ߂l   : Ȃ                                                         
 *  l     : Ȃ                                                         
 ****************************************************************************/
static void __exit rstinfo_exit(void)
{
    /* foCX̍폜 */
    misc_deregister(&t_rstinfo_miscdev);
}

/* J[lW[̏^ */
module_init(rstinfo_init);
module_exit(rstinfo_exit);

