/* Copyright (c) 2010, Code Aurora Forum. All rights reserved.
 * This software is contributed or developed by KYOCERA Corporation.
 * (C) 2011 KYOCERA Corporation
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License version 2 and
 * only version 2 as published by the Free Software Foundation.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA
 * 02110-1301, USA.
 */

#include <linux/init.h>
#include <linux/wait.h>
#include <linux/gpio.h>
#include <mach/pmic.h>
#include <mach/msm_qdsp6_audio.h>
#include <asm/string.h>
#include <asm/mach-types.h>

#ifdef CONFIG_KYOCERA_ORIGINAL_FEATURE
#include <linux/delay.h>
#include <mach/debug_mm.h>
#include "mach/hs_io_ctl_a.h"
#include "mach/msm_battery.h"
#endif /* CONFIG_KYOCERA_ORIGINAL_FEATURE */

#define GPIO_HEADSET_AMP 157
#define GPIO_SPEAKER_AMP 39
#define GPIO_HEADSET_SHDN_N 48

void analog_init(void)
{
	/* stereo pmic init */
	pmic_spkr_set_gain(LEFT_SPKR, SPKR_GAIN_PLUS12DB);
	pmic_spkr_set_gain(RIGHT_SPKR, SPKR_GAIN_PLUS12DB);
	pmic_mic_set_volt(MIC_VOLT_1_80V);

	if (machine_is_qsd8x50a_st1_5()) {
		gpio_set_value(GPIO_SPEAKER_AMP, 0);
		gpio_set_value(GPIO_HEADSET_SHDN_N, 0);
	} else {
		gpio_direction_output(GPIO_HEADSET_AMP, 1);
		gpio_set_value(GPIO_HEADSET_AMP, 0);
	}
}

void analog_headset_enable(int en)
{
	/* enable audio amp */
	if (machine_is_qsd8x50a_st1_5())
		gpio_set_value(GPIO_HEADSET_SHDN_N, !!en);
	else
		gpio_set_value(GPIO_HEADSET_AMP, !!en);
}

void analog_speaker_enable(int en)
{
	struct spkr_config_mode scm;
	memset(&scm, 0, sizeof(scm));

	if (en) {
		scm.is_right_chan_en = 1;
		scm.is_left_chan_en = 1;
		scm.is_stereo_en = 1;
		scm.is_hpf_en = 1;
		pmic_spkr_en_mute(LEFT_SPKR, 0);
		pmic_spkr_en_mute(RIGHT_SPKR, 0);
		pmic_set_spkr_configuration(&scm);
		pmic_spkr_en(LEFT_SPKR, 1);
		pmic_spkr_en(RIGHT_SPKR, 1);

		/* Enable Speaker Amplifier */
		if (machine_is_qsd8x50a_st1_5()) {
			pmic_secure_mpp_control_digital_output(
					PM_MPP_21, PM_MPP__DLOGIC__LVL_VDD,
					PM_MPP__DLOGIC_OUT__CTRL_HIGH);
			gpio_set_value(GPIO_SPEAKER_AMP, !!en);
		}
		
		/* unmute */
		pmic_spkr_en_mute(LEFT_SPKR, 1);
		pmic_spkr_en_mute(RIGHT_SPKR, 1);
	} else {
		pmic_spkr_en_mute(LEFT_SPKR, 0);
		pmic_spkr_en_mute(RIGHT_SPKR, 0);

		/* Disable Speaker Amplifier */
		if (machine_is_qsd8x50a_st1_5()) {
			gpio_set_value(GPIO_SPEAKER_AMP, !!en);
			pmic_secure_mpp_control_digital_output(
					PM_MPP_21, PM_MPP__DLOGIC__LVL_VDD,
					PM_MPP__DLOGIC_OUT__CTRL_LOW);
		}

		pmic_spkr_en(LEFT_SPKR, 0);
		pmic_spkr_en(RIGHT_SPKR, 0);

		pmic_set_spkr_configuration(&scm);
	}
}

void analog_mic_enable(int en)
{
	pmic_mic_en(en);
}

#ifdef CONFIG_KYOCERA_ORIGINAL_FEATURE
void analog_ext_init
(
void
)
{
#if defined(CONFIG_TARGET_PROTOTYPE_WS2)
  switch(HS_INTER_HW_VER)
  {
    case HS_INTER_HW_VER_WS21:
      pmic_mic_set_volt(MIC_VOLT_2_00V);
      break;

    case HS_INTER_HW_VER_WS22:
    case HS_INTER_HW_VER_WS23:
    case HS_INTER_HW_VER_WS24:
    case HS_INTER_HW_VER_WS31:
      HS_VDD_MIC_EN(HS_VDD_MIC_DISABLE,HS_VDD_MIC_GID_MEDIA);
      break;

    default:
      pmic_mic_set_volt(MIC_VOLT_2_00V);
      HS_VDD_MIC_EN(HS_VDD_MIC_DISABLE,HS_VDD_MIC_GID_MEDIA);
      break;
  }
#endif

  HS_A_SPK_ON1_LO();
  HS_A_SPK_ON2_LO();

  msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_OFF, false );
}

void analog_ext_speaker_enable
(
int en
)
{
  if (en)
  {
    HS_SPEAKER_POWER_ON();
    HS_A_SPK_ON1_HI();
    HS_A_SPK_ON2_HI();
    msleep(10);	
    HS_A_SPK_ON1_LO();
    msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_SPEAKER, true );
    pr_info("analog audio: speaker on\n");
  }
  else
  {
    HS_A_SPK_ON2_LO();
    HS_A_SPK_ON1_LO();
    HS_SPEAKER_POWER_OFF();
    msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_SPEAKER, false );
    pr_info("analog audio: speaker off\n");
  }
}

void hosei_headset_enable
(
int en
)
{
  if (en)
  {
    msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_HEADSET, true );
    pr_info("analog audio: headset on\n");
  }
  else
  {
    msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_HEADSET, false );
    pr_info("analog audio: headset off\n");
  }
}

void hosei_receiver_enable
(
int en
)
{
  if (en)
  {
    msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_RECEIVER, true );
    pr_info("analog audio: receiver on\n");
  }
  else
  {
    msm_batt_oem_update_dev_info_snd( DEV_INFO_SND_RECEIVER, false );
    pr_info("analog audio: receiver off\n");
  }
}

void analog_int_mic_enable
(
int en
)
{

#if defined(CONFIG_TARGET_PROTOTYPE_WS2)
  switch(HS_INTER_HW_VER)
  {
    case HS_INTER_HW_VER_WS21:
      pmic_mic_en( (uint)en );
      pr_info("analog audio: pmic mic bias (%d)\n", en);
      break;

    case HS_INTER_HW_VER_WS22:
    case HS_INTER_HW_VER_WS23:
    case HS_INTER_HW_VER_WS24:
    case HS_INTER_HW_VER_WS31:
      if (en)
      {
        HS_VDD_MIC_EN(HS_VDD_MIC_ENABLE,HS_VDD_MIC_GID_MEDIA);
      }
      else
      {
        HS_VDD_MIC_EN(HS_VDD_MIC_DISABLE,HS_VDD_MIC_GID_MEDIA);
      }
      pr_info("analog audio: vdd mic bias(%d)\n", en);
      break;

    default:
      if (en)
      {
        HS_VDD_MIC_EN(HS_VDD_MIC_ENABLE,HS_VDD_MIC_GID_MEDIA);
      }
      else
      {
        HS_VDD_MIC_EN(HS_VDD_MIC_DISABLE,HS_VDD_MIC_GID_MEDIA);
      }
      pmic_mic_en( (uint)en );
      pr_info("analog audio: vdd/pmic mic bias (%d)\n", en);
      break;
  }
#endif

}

void analog_ext_mic_enable
(
int en
)
{
  pmic_mic_en( (uint)en );
  pr_info("analog audio: mic bias(%d)\n", en);
}
#endif /* CONFIG_KYOCERA_ORIGINAL_FEATURE */

#ifdef CONFIG_KYOCERA_ORIGINAL_FEATURE
static struct q6audio_analog_ops ops = {
  .init           = analog_ext_init,
  .speaker_enable = analog_ext_speaker_enable,
  .headset_enable = hosei_headset_enable,
  .receiver_enable= hosei_receiver_enable,
  .int_mic_enable = analog_int_mic_enable,
  .ext_mic_enable = NULL,
};
#else  /* CONFIG_KYOCERA_ORIGINAL_FEATURE */
static struct q6audio_analog_ops ops = {
	.init = analog_init,
	.speaker_enable = analog_speaker_enable,
	.headset_enable = analog_headset_enable,
	.int_mic_enable = analog_mic_enable,
	.ext_mic_enable = analog_mic_enable,
};
#endif /* CONFIG_KYOCERA_ORIGINAL_FEATURE */

static int __init init(void)
{
	q6audio_register_analog_ops(&ops);
	return 0;
}

device_initcall(init);
